﻿########################################
# Zusammenfügen von CSV-Messdatein     #
# PM, Version 1.0 17.04.2021           #
########################################
#
# Voraussetzungen:
#
# * Alle CSV-Dateien haben die gleiche Zeilenanzahl
# * Werte in gleichen Zeilen wurden zum gleichen Zeitpunkt gemessen
# 

#Set-Location I:\Hartmut\Wetter\Archiv_original_csv_Dateien\030421\gekuerzteDateien
Set-Location C:\Users\ad\Documents\Hartmut

$sensorliste  = 'sensoren.csv'
$ausgabedatei = 'komplett.csv'

#
# Einlesen der Sensorliste
#
# Dateibeispiel:
#
# Sensordatei,Sensorname
# 202103A.CSV,A
# 2021CH1A.CSV,B
# 2021CH2A.CSV,C
# 2021CH3A.CSV,D
#
$Sensoren = import-csv $sensorliste -delimiter ","

function spalten_namen( $spaltenkopf, $sname )
{   
  #
  # Die Funktion hängt an die Spaltennamen die 
  # Sensornamen, getrennt durch : an. Damit wird sichergestellt, 
  # dass die Spaltennamen eindeutig sind
  # Bsp.: aus Temp (°C) des Sensors xy wird Temp (°C):xy
  # 
    $k  = $spaltenkopf -split ","
    $t  = $k -join ":$sname,"
    return $t + ":$sname"
}

#
# Zusammenfügen der CSV-Dateien
#
for ( $i=0; $i -lt $sensoren.count; $i++ ) 
{ 
   # Die in der Sensorliste aufgeführten CSV-Dateien werden
   # nacheinander eigelesen.
   # Die Inhalten werden in $gesamt zusammengeführt
   #  
   $aktcsv    = Get-Content -Encoding UTF8 $sensoren[$i].Sensordatei
   $aktcsv[0] = spalten_namen -spaltenkopf $aktcsv[0] -sname $sensoren[$i].Sensorname 

   if ( $i -eq 0 )
   {
     # Die ersten CSV-Datei wird incl. Zeitspalte übernommen
     $gesamt = $aktcsv;
   } 
   else
   {
     # bei den übrigen Dateien wir die Zeitspalte entfernt
     for ( $j=0; $j -lt $aktcsv.Count; $j++ )
     {
       # Abtrennem des Wertes der esten Spalte
       $z,$t        = $aktcsv[$j] -split ","
       # der Wert der ersten Spalte befindet sich in $z, 
       # die Werte der übrigen Spalten befinden sich in $t
       #
       # Zusammenfügen der Werte in $t zu einem String.
       # Als Trennzeichen wird Komma verwendet.   
       $werte       = $t -join ','
       # Das Ergebnis wird an die aktuelle Zeile der Gesamtdatei
       # getrennt durch Komma angehängt.
       $gesamt[$j] += "," + $werte
     }
   }
}

#$gesamt
# Speichern der Gesamtdatei
$gesamt | convertFrom-csv -Delimiter ","  | export-csv $ausgabedatei -delimiter "," -NoTypeInformation -Encoding UTF8